/**
 * Processes raw air quality observations from WFS into standardized format
 * @async
 * @param {Object} context - Fission execution context
 * @returns {Object} object containing:
 * @property {number} status - HTTP status code
 * @property {string} body - JSON string of processed observations
 */
module.exports = async function (context) {
  // Log raw input for debugging purposes
  console.log('Processing air quality observation:', 
    JSON.stringify(context.request.body, null, 2));

  // Transform GeoJSON features into our internal observation format
  const processedData = context.request.body.features.map(feat => ({
    stationid: feat.properties.site_id,      // Unique station identifier
    name: feat.properties.site_name,        // Human-readable station name
    geo: [                                  // GPS coordinates [long, lat]
      feat.properties.longitude,
      feat.properties.latitude
    ],
    timestamp: feat.properties.time_stamp,  // ISO 8601 observation time
    pm10: feat.properties.pm10,             // PM10 particulate matter (μg/m³)
    pm2p5: feat.properties.pm2p5,           // PM2.5 particulate matter (μg/m³)
    ozone: feat.properties.ozone            // O3 concentration (ppb)
  }));

  return {
    status: 200,
    body: JSON.stringify(processedData, null, 2)
  };
}
