import logging
import json
import requests
from flask import current_app
from typing import Dict, Any, Optional


def main() -> str:
    """Harvests weather observations from Bureau of Meteorology API.

    Performs:
    - HTTP GET request to BOM's public weather observations API
    - Logs successful harvest operation
    - Enqueues raw data to message queue via Fission router

    Returns:
        str: 'OK' on successful harvest and enqueue

    Note:
        Uses IDV60901.95936 product code for Mildura Airport observations
    """
    data: Dict[str, Any] = requests.get(
        'http://reg.bom.gov.au/fwo/IDV60901/IDV60901.95936.json'
    ).json()
    
    current_app.logger.info(
        f'Harvested {len(data["observations"]["data"])} weather observations'
    )

    response: Optional[requests.Response] = requests.post(
        url='http://router.fission/enqueue/weather',
        headers={'Content-Type': 'application/json'},
        data=json.dumps(data)
    )
    
    return 'OK'
