/**
 * Transforms raw BOM weather data into standardized observation format
 * @async
 * @param {Object} context - Fission execution context
 * @returns {Object} Processed response 
 */
module.exports = async function (context) {
  // Extract raw observations from request body
  const rawObservations = context.request.body.observations.data;

  // Transform to standardized schema
  const processed = rawObservations.map(obs => ({
    timestamp: obs.local_date_time_full,  // ISO 8601 timestamp
    station: {
      id: obs.aifstime_utc,
      name: obs.name
    },
    temperature: obs.air_temp,          // Celsius
    humidity: obs.rel_hum,              // Percentage
    wind_speed: obs.wind_spd_kmh,       // km/h
    rainfall: obs.rain_trace            // mm
  }));

  // Log transformation metrics
  console.log(`Processed ${processed.length} observations`);
  
  return {
    status: 200,
    body: JSON.stringify(processed, null, 2)
  };
}
