# Software Stack Installation

## Pre-requirements

- OpenStack clients >= 5.8.x ([Installation instructions](https://docs.openstack.org/newton/user-guide/common/cli-install-openstack-command-line-clients.html)).
  > Not: Please ensure the following Openstack clients are installed: `python-cinderclient`, `python-keystoneclient`, `python-magnumclient`, `python-neutronclient`, `python-novaclient`, `python-octaviaclient`. See: [Install the OpenStack client](https://docs.openstack.org/newton/user-guide/common/cli-install-openstack-command-line-clients.html).
- JQ >= 1.7.x ([Installation instructions](https://jqlang.github.io/jq/download/)).
- kubectl >= 1.30.0 and < 1.33.0 ([Installation instructions](https://kubernetes.io/docs/tasks/tools/)).
- Helm >= 3.17.x ([Installation instructions](https://helm.sh/docs/intro/install/)).
- MRC project with enough resources to create a Kubernetes cluster.

Open a shell and move to the directory of the repository that contains this README file.

## Client Configuration

1. Log in to the [MRC Dashboard](https://dashboard.cloud.unimelb.edu.au/auth/login/?next=/) with your University of Melbourne credentials and select the project you want to use.
   ![Select Project](./screenshots/mrc_01.jpg)

2. Download the OpenStack RC file from the `User` menu.
   ![Download RC file](./screenshots/mrc_02.jpg)

3. Obtain the Openstack password from `User` -> `Settings` menu, click on `Reset Password` on the left and save the password in a safe place.
   ![Reset Password](./screenshots/mrc_03.jpg)

4. Source the OpenStack RC file downloaded in step 2 in your terminal and enter the password obtained in step 3 when prompted.

> Note: Password will not be displayed on the screen when typed.

```shell
source ./<your project name>-openrc.sh
```

5. Click `Project` -> `Compute` -> `Key Pairs` -> `Create Key Pair` and create a new key pair named `mykeypair` (replace `mykeypair` with the name you prefer). Keep the private key file downloaded (e.g. `mykeypair.pem`) in a safe place.
   ![Create Key Pair (1/2)](./screenshots/mrc_04.jpg)
   ![Create Key Pair (2/2)](./screenshots/mrc_05.jpg)

6. All team members must have their key pairs created and the public key file added to the project (see the previous step).

## Kubernetes Cluster Provisioning

- Create a Kubernetes cluster named "elastic" with the newly created template (1 master node and 3 worker nodes).

```shell
openstack coe cluster create\
  --cluster-template "kubernetes-v1.31.1-melbourne-qh2-uom-v4" \
  --node-count 3 \
  --master-count 1 \
  --master-flavor "uom.mse.2c9g"\
  --flavor "uom.mse.2c9g"\
  comp90024
```

> Note: do not change the cluster name or the number of master nodes
>
> Note: up to four worker nodes can be created, but we have to leave some spare capacity for replacing a failing node.

- Check whether the cluster has been created healthy (it may take more than 15 minutes).

```shell
openstack coe cluster show comp90024 --fit-width
```

> Note: `status` should be `CREATE_COMPLETE`, `health_status` should be `HEALTHY` and `coe_version` should be `v1.31.1`

## Accessing the Kubernetes Cluster

- Create the Kubernetes configuration file to access the cluster. This command will create a file named `config` in the current directory.

```shell
openstack coe cluster config comp90024
```

- Move the config file to `~/.kube/config` (alternatively, set the `KUBECONFIG` environment variable).

> Note: If you already have a `~/.kube/config` file, back it up before executing the following command.

```shell
mv config ~/.kube/config
chmod 600 ~/.kube/config
```

Note: once the `~/.kube/config` file has been created by the team member that has created the Kubernetes cluster, it can be
shared across the other team members.

- Check the cluster nodes:

```shell
kubectl get nodes
```

This command should return one master node and the required number of worker nodes.

```shell
NAME                                                STATUS   ROLES           AGE   VERSION
comp90024-5vcaf7zpapei-control-plane-qvv2r          Ready    control-plane   24m   v1.31.1
comp90024-5vcaf7zpapei-default-worker-7c8wp-knt6k   Ready    <none>          20m   v1.31.1
comp90024-5vcaf7zpapei-default-worker-7c8wp-lkxmm   Ready    <none>          20m   v1.31.1
comp90024-5vcaf7zpapei-default-worker-7c8wp-nj8zx   Ready    <none>          20m   v1.31.1
```

## Provision of a performance storage class

The following command creates a new storage class named `perfretain` that uses the `performance` storage type
with the `retain` reclaim policy (to avoid losing data when ElasticSearch is uninstalled).
```shell
kubectl apply -f ./installation/storage-class.yaml
```

## ElasticSearch cluster installation

Set the ElasticSearch version to be used, then install ElasticSearch:

```shell
export ES_VERSION="8.5.1"
helm repo add elastic https://helm.elastic.co
helm repo update
helm upgrade --install \
  --version=${ES_VERSION} \
  --create-namespace \
  --namespace elastic \
  --set replicas=2 \
  --set secret.password="elastic"\
  --set volumeClaimTemplate.resources.requests.storage="100Gi" \
  --set volumeClaimTemplate.storageClassName="perfretain" \
  elasticsearch elastic/elasticsearch
```

> NOTES:
>
> - The ElasticSearch version used does not include ES|QL as a stable release;
> - By default each ElasticSearch node has 30GB of storage;
> - The number of nodes is set by the `replicas` parameter. not to be confused with the "shard replicas" (copies of a shard);
> - The number of replicas (nodes) that can be used in the cluster is limited by the number of nodes in the cluster and by the Kibana deployment that needs a node for itself.
> - Passing an unsafe password as `secret.password` to Helm is a security risk and it's done here for the sake of simplicity: in a production environment the password must be randomly generated and of suitable length (secure passwords can be generated with the Linux command `pwgen -n 32`).

Check all ElasticSearch pods are running before proceeding:

```shell
kubectl get pods -l release=elasticsearch -n elastic --watch
```

This command will watch the pods' status. You can use `Ctrl + C` to stop watching once you see the pods are in `Running` state.

```shell
NAME                     READY   STATUS     RESTARTS   AGE
elasticsearch-master-0   0/1     Init:0/1   0          20s
elasticsearch-master-1   0/1     Init:0/1   0          20s
elasticsearch-master-0   0/1     Init:0/1   0          59s
elasticsearch-master-1   0/1     Init:0/1   0          59s
elasticsearch-master-1   0/1     PodInitializing   0          60s
elasticsearch-master-0   0/1     PodInitializing   0          61s
elasticsearch-master-1   0/1     Running           0          61s
elasticsearch-master-0   0/1     Running           0          62s
```

## Kibana installation

```shell
helm upgrade --install \
  --version=${ES_VERSION} \
  --namespace elastic \
  -f ./installation/kibana-values.yaml \
  kibana elastic/kibana
```

Check all ElasticSearch pods are running before proceeding:

```shell
kubectl get pods -l release=kibana -n elastic --watch
```

This command will watch the pods' status. You can use `Ctrl + C` to stop watching once you see the pods are in `Running` state.

Check all services are created before proceeding.

```shell
kubectl get service -n elastic
```

This command will list all service in the Kubernetes cluster.

```shell
NAME                            TYPE        CLUSTER-IP     EXTERNAL-IP   PORT(S)             AGE
elasticsearch-master            ClusterIP   10.254.75.27   <none>        9200/TCP,9300/TCP   17h
elasticsearch-master-headless   ClusterIP   None           <none>        9200/TCP,9300/TCP   17h
kibana-kibana                   ClusterIP   10.254.50.97   <none>        5601/TCP            17h
```

## Fission Deployment

```shell
export FISSION_VERSION='1.21.0'
kubectl create -k "github.com/fission/fission/crds/v1?ref=v${FISSION_VERSION}"
helm repo add fission-charts "https://fission.github.io/fission-charts/"
helm repo update
helm upgrade fission fission-charts/fission-all --install\
  --version v${FISSION_VERSION}\
  --namespace fission \
  --create-namespace\
  --set routerServiceType='ClusterIP'\
  --set persistence.storageClass='perfretain'
```

> Note: for detailed instructions see [here](https://fission.io/docs/installation/)

After the installation, wait for all pods to have started. (This command will watch the pods' status. You can use `Ctrl + C` to stop watching once you see the pods are in `Running` state.)

```shell
kubectl get pods -n fission --watch
```

## Fission Client

### Mac & Linux

```shell
OS=$(uname -s | tr '[:upper:]' '[:lower:]')
curl -Lo fission https://github.com/fission/fission/releases/download/v${FISSION_VERSION}/fission-v${FISSION_VERSION}-${OS}-amd64 \
   && chmod +x fission && sudo mv fission /usr/local/bin/
```
> Note: on Apple M1-M4 microprocessors the architecture must be `arm64` and not `amd64`


### Mac (brew)

```shell
brew tap xxxbrian/tap
brew install fission-cli
```

### Windows

For Windows, please use the Linux binary on WSL.

### asdf

Alternatively, you can install the Fission CLI using asdf:

```shell
asdf plugin add fission
asdf install fission $FISSION_VERSION
asdf set -u fission $FISSION_VERSION
```

### Validate the installation

```shell
fission check
```

> Note: When running Fission CLI for the first time on MacOS, you may see a security warning stating that "Apple could not verify xxxx". This is normal for third-party applications. To resolve this:
>
> 1. When you see the warning popup, click 'Cancel' or 'Done'
> 2. Go to System Settings → Privacy & Security
> 3. Scroll down to the Security section where you'll see a message about fission-cli being blocked
> 4. Click 'Allow Anyway' to permit the application to run
> 5. When you run the application again, click 'Open Anyway' in the final confirmation dialog

## Stack installation test

The following command creates and test a function named `health` that returns the status of the ElasticSearch cluster:

```shell
(
  cd fission
  fission env create --name python --image fission/python-env --builder fission/python-builder
  fission function create --name health --env python --code ./functions/health.py
  fission function test --name health | jq '.'
)
```

If the above command returns a JSON object with the ElasticSearch cluster status, the installation is successful, and
the Fission function and environment can be deleted:

```shell
fission function delete --name health
fission env delete --name python
```

We now know that the ElasticSearch cluster is up and running, and the Fission installation is successful, let's try Kibana:

```shell
kubectl port-forward service/kibana-kibana -n elastic 5601:5601
```

(It uses t a Kubernetes port-forward of a service to access the Kibana dashboard on `http://localhost:5601`.)

> Note: Do not close the terminal session that is running the port-forwarding.

## Removal of the software stack

THIS SHOULD BE DONE ONLY IN CASE OF A SERIOUS MISTAKE THAT PREVENTS USE OF THE CLUSTER.

### Fission uninstallation

```shell
helm uninstall fission --namespace fission
```

### ElasticSearch Cluster Removal

```shell
helm uninstall kibana -n elastic
helm uninstall elasticsearch -n elastic
```

### Kubernetes Volume removal

```shell
kubectl get pvc -o name -n elastic | xargs -i kubectl delete {} -n elastic
kubectl get pvc -o name -n fission | xargs -i kubectl delete {} -n fission
kubectl get pv -o name | xargs -i kubectl delete {}
```

### Storage class removal

```shell
kubectl delete storageclass perfretain
```

### Kubernetes Cluster Removal

```shell
openstack coe cluster delete comp90024
```

Wait until the volumes became `available` and delete them:

```shell
openstack volume list -f value -c ID | xargs -i openstack volume delete {}
```
