from flask import request, current_app
import requests, logging, json
from typing import Optional, Tuple, Any

def config(k: str) -> str:
    """Read configuration parameter from default configs.
    
    Args:
        k: Configuration key filename to read
        
    Returns:
        String contents of the configuration file
    """
    with open(f'/configs/default/parameters/{k}', 'r') as f:
        return f.read()


def main() -> Tuple[Optional[Any], int]:
    """Retrieve course information from Elasticsearch.
    
    Returns:
        Tuple containing:
        - Parsed JSON response (or None on error)
        - HTTP status code
    """
    es_url = config("ES_URL")
    es_db = config("ES_DATABASE")
    es_auth = (config("ES_USERNAME"), config("ES_PASSWORD"))
    
    r = requests.get(
        f'{es_url}/{es_db}',
        verify=False,
        auth=es_auth
    )
    try:
        return r.json(), r.status_code
    except json.JSONDecodeError:
        return None, r.status_code
