from flask import Request, current_app
import requests, logging, json
from typing import Dict, Any, Tuple

def config(k: str) -> str:
    """Read configuration parameter from default configs.
    
    Args:
        k: Configuration key filename to read
        
    Returns:
        String contents of the configuration file
    """
    with open(f'/configs/default/parameters/{k}', 'r') as f:
        return f.read()


def main() -> Tuple[Dict[str, Any], int]:
    """Recreate Elasticsearch index with fresh schema.
    
    Steps:
    1. Delete existing index if present
    2. Create new index using schema from config
    3. Return final operation status
    
    Returns:
        Tuple containing:
        - JSON response from ES
        - HTTP status code
    """
    # Delete existing index
    delete_response = requests.delete(
        f'{config("ES_URL")}/{config("ES_DATABASE")}',
        verify=False,
        auth=(config("ES_USERNAME"), config("ES_PASSWORD"))
    )
    
    # Recreate index with schema
    create_response = requests.put(
        f'{config("ES_URL")}/{config("ES_DATABASE")}',
        verify=False,
        auth=(config("ES_USERNAME"), config("ES_PASSWORD")),
        headers={'Content-type': 'application/json'},
        data=config("ES_SCHEMA")
    )
    
    return create_response.json(), create_response.status_code
