from flask import Request, current_app
from typing import Tuple, Dict, Any
import requests
import logging
import json
from Commons import Commons
from ESCourse import ESCourse

ResponseTuple = Tuple[Dict[str, Any], int]

def main(request: Request) -> ResponseTuple:
    """
    Search all course documents in Elasticsearch.

    Constructs a search query to retrieve:
    - All documents of type 'course'
    - Only specified fields (configured in ESCourse)

    Args:
        request (Request): The Flask request object.

    Returns:
        Tuple containing:
        - Search results JSON
        - HTTP status code

    Raises:
        Exception: Logs error and returns 500 on failure
    """
    try:
        commons: Commons = Commons()
        es_course: ESCourse = ESCourse(commons, request)
        
        # Execute search query
        response: requests.Response = requests.post(
            commons.search_url(),
            verify=False,
            auth=commons.auth(),
            headers={'Content-type': 'application/json'},
            data=es_course.all()
        )
        return response.json(), response.status_code

    except Exception as e:
        current_app.logger.error(str(e))
        return {'message': f'Error: {str(e)}'}, 500
