from flask import Request, current_app
from typing import Tuple, Dict, Any
import requests, logging, json
from Commons import Commons
from ESStudent import ESStudent

def main(request: Request) -> Tuple[Dict[str, Any], int]:
    """Handle student document CRUD operations via HTTP methods.

    Supports:
    - GET: Retrieve student document
    - PUT: Update/Create student document
    - DELETE: Remove student document

    Returns:
        Tuple containing response JSON and HTTP status code

    Raises:
        Exception: Logs errors and returns 500 on unexpected failures
    """
    try:
        student = ESStudent(Commons(), request)

        if request.method == 'GET':
            r = requests.get(student.url(), verify=False, auth=Commons().auth())
            return r.json(), r.status_code

        elif request.method == 'PUT':
            r = requests.get(student.url(), verify=False, auth=Commons().auth())

            params = {'if_seq_no': r.json()['_seq_no'], 'if_primary_term': r.json()['_primary_term']} \
                if r.status_code == 200 else {}

            r = requests.put(
                student.url(),
                verify=False,
                auth=Commons().auth(),
                headers={'Content-type': 'application/json'},
                data=json.dumps(student.add_fields(request.json)),
                params=params
            )
            return r.json(), r.status_code

        elif request.method == 'DELETE':
            r = requests.get(student.url(), verify=False, auth=Commons().auth())
            if r.status_code != 200:
                return r.json(), r.status_code

            r = requests.delete(
                student.url(),
                verify=False,
                auth=Commons().auth(),
                params={'if_seq_no': r.json()['_seq_no'], 'if_primary_term': r.json()['_primary_term']}
            )
            return r.json(), r.status_code

    except Exception as e:
        current_app.logger.error(e)
        return {'message': f'Error {e}'}, 500

    return {'message': 'Method not allowed'}, 405
