import unittest
import json
import time
import sys
from typing import Dict, Any
from unittest.mock import MagicMock

sys.path.append('./functions/library')
from Commons import Commons
from ESCourse import ESCourse
from ESStudent import ESStudent
from flask import Request, current_app


class TestUnit(unittest.TestCase):
    """Unit test suite for core Elasticsearch document handling components.

    Tests focus on:
    - URL generation logic validation
    """

    def setUp(self) -> None:
        """Initialize mock dependencies for test isolation."""
        self.mock_commons: MagicMock = MagicMock(autospec=Commons)
        self.mock_commons.config = MagicMock(return_value='x')
        
        self.mock_request: MagicMock = MagicMock(autospec=Request)
        self.mock_request.headers = {
            'X-Fission-Params-Courseid': 'comp90024',
            'X-Fission-Params-Studentid': '123'
        }

    def test_escourse_url_generation(self) -> None:
        """Validate ESCourse URL construction with course ID from headers.

        Verifies:
        - Proper concatenation of ES base URL and index name
        """
        test_escourse = ESCourse(self.mock_commons, self.mock_request)
        self.assertEqual(test_escourse.url(), 'x/x/_doc/course_comp90024')

    def test_esstudent_url_generation(self) -> None:
        """Validate ESStudent URL construction with student ID from headers.

        Verifies:
        - Correct student document path formatting
        """
        test_esstudent = ESStudent(self.mock_commons, self.mock_request)
        self.assertEqual(test_esstudent.url(), 'x/x/_doc/student_123')


if __name__ == '__main__':
    unittest.main()
